/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

/**
 * Represents a business entity in the registry. This class could be used to
 * store both commercial business entities and non-profit organization
 * informations, eventually through subclassing.
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`business`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class Business extends Entity {

	private static final long serialVersionUID = 2400674709089161829L;

	/**
	 * The main address of the business, usually defined as <i>legal situs </i>.
	 * 
	 * @directed true
	 */
	private MailAddress legalSitus;

	/**
	 * Identifing code usually used as unique financial reference to the
	 * business.
	 */
	private String idCode;

	/**
	 * Returns the main address of the business, usually defined as <i>legal
	 * situs </i>.
	 * 
	 * @return the main address of the business.
	 * @hibernate.many-to-one column="`legalSitus`"
	 */
	public MailAddress getLegalSitus() {
		return legalSitus;
	}

	/**
	 * Sets the main address of the business, usually defined as <i>legal situs
	 * </i>.
	 * 
	 * @param legalSitus the main address of the business.
	 */
	public void setLegalSitus(MailAddress legalSitus) {
		this.legalSitus = legalSitus;
	}

	/**
	 * Returns the identifing code usually used as unique financial reference to
	 * the business.
	 * 
	 * @return returns the identifing code.
	 * @hibernate.property column="`idCode`"
	 */
	public String getIdCode() {
		return idCode;
	}

	/**
	 * Sets the identifing code usually used as unique financial reference to
	 * the business.
	 * 
	 * @param idCode the identifing code.
	 */
	public void setIdCode(String idCode) {
		this.idCode = idCode;
	}
}
