/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

/**
 * Represents an electronic mail address used for email delivery.
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`email-address`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class EmailAddress extends Address {

	private final static long serialVersionUID = -419769087017551138L;

	/**
	 * The email address.
	 */
	private String email;


	/**
	 * Returns the email address.
	 * 
	 * @hibernate.property column="`email`" length="200"
	 * @return returns the email.
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * Sets the email address.
	 * 
	 * @param email the email to set.
	 */
	public void setEmail(String email) {
		this.email = email;
	}

    /**
     * @see net.smartlab.web.registry.Address#getFormattedString()
     */
    public String getFormattedString() {
        return email;
    }
}