/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

import java.util.Collection;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;

import net.smartlab.web.BusinessObjectFactory;
import net.smartlab.web.DAOException;

/**
 * TODO documentation
 * @author rlogiacco
 */
public class EntityFactory extends BusinessObjectFactory {

	/**
	 * Singleton instance statically initialized.
	 */
	private final static EntityFactory instance = new EntityFactory();

	/**
	 * Blocks class instantiation to enforce the singleton pattern.
	 */
	private EntityFactory() {
		super();
	}

	/**
	 * @see net.smartlab.web.BusinessObjectFactory#getMappedClass()
	 */
	public Class getMappedClass() {
		return Entity.class;
	}

	/**
	 * @return Returns the singleton instance.
	 */
	public static EntityFactory getInstance() {
		return instance;
	}

	/**
	 * TODO documentation
	 * @param entity
	 * @param strategy
	 * @return
	 * @throws DAOException
	 */
	public Collection listDuplicates(Entity entity, DeduplicationStrategy strategy) throws DAOException {
		if (logger.isDebugEnabled()) {
			logger.debug("listDuplicates(entity = " + entity + ") - start");
		}
		try {
			Criteria criteria = strategy.getCriteria(super.current().createCriteria(Entity.class), strategy.digest(entity));
			// FIXME
			//criteria.add(Expression.eq("owner", super.getUser()));
			return criteria.list();
		} catch (HibernateException he) {
			logger.error("listDuplicates(entity = " + entity + ") - error", he);
			throw new DAOException("persistence.error.search", he);
		}
	}
}
