/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

import java.util.Set;

/**
 * TODO documentation
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`list`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class List extends Entry {

	private final static long serialVersionUID = 4149872229997965218L;

	private String description;

	/**
	 * @link aggregation <{Entry}>
	 * @directed directed
	 * @supplierCardinality 0..*
	 */
	private Set entries;

	/**
	 * Number of entities members of this group.
	 */
	private int size;


	/**
	 * Returns the group description.
	 * 
	 * @return returns the group <code>description</code>.
	 * @hibernate.property column="description" type="text"
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Sets the group description.
	 * 
	 * @param description the group description to set.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return returns the members.
	 * @hibernate.set lazy="true" schema="registry" table="`list_entry`"
	 *                sort="natural"
	 * @hibernate.collection-key column="`list`"
	 * @hibernate.collection-many-to-many column="`entry`"
	 *                                    class="net.smartlab.web.registry.Entry"
	 */
	public Set getEntries() {
		return entries;
	}

	/**
	 * Sets the group members.
	 * 
	 * @param entries the members to set.
	 */
	protected void setEntries(Set entries) {
		this.entries = entries;
	}

	/**
	 * Ensures the group has no members.
	 */
	public void clearEntries() {
		entries.clear();
	}

	/**
	 * TODO documentation
	 * @param entry
	 */
	public void add(Entry entry) {
		entries.add(entry);
	}

	/**
	 * TODO documentation
	 * @param entry
	 */
	public void remove(Entry entry) {
		entries.remove(entry);
	}

	/**
	 * Returns the size of the list in terms of number of contained entries. The
	 * returned value will not count sub lists recursively.
	 * 
	 * @return returns the list size.
	 * @hibernate.property insert="false" update="false" formula="( select
	 *                     count(*) from `registry`.`list_entry` LE where
	 *                     LE.`list`=id )"
	 */
	public int getSize() {
		return size;
	}

	/**
	 * Sets the list size.
	 * 
	 * @param size the list size.
	 */
	protected void setSize(int size) {
		this.size = size;
	}
}