/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

/**
 * Represents a fully qualified mail address usable for postal delivery
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`mail-address`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class MailAddress extends Address {

	private final static long serialVersionUID = -678094002234942516L;

	/**
	 * An optional description for delivery purposes.
	 */
	private String at;
	/**
	 * The fully qualified city.
	 * 
	 * @directed true
	 * @stereotype component
	 */
	private Locality locality;
	/**
	 * An optional name to describe an additional subdivision of the city.
	 */
	private String hamlet;
	/**
	 * The delivery code used in local area identifications.
	 */
	private String zipCode;
	/**
	 * The street.
	 */
	private String street;
	/**
	 * The string identifing the street subdivision.
	 */
	private String civic;

	/**
	 * TODO documentation
	 * @return returns the at.
	 * @hibernate.property column="`at`" length="250"
	 */
	public String getAt() {
		return at;
	}

	/**
	 * TODO documentation
	 * @param at the at to set.
	 */
	public void setAt(String at) {
		this.at = at;
	}

	/**
	 * TODO documentation
	 * @return returns the civic.
	 * @hibernate.property column="`civic`" length="50"
	 */
	public String getCivic() {
		return civic;
	}

	/**
	 * TODO documentation
	 * @param civic the civic to set.
	 */
	public void setCivic(String civic) {
		this.civic = civic;
	}

	/**
	 * TODO documentation
	 * @hibernate.component
	 * @return returns the locality.
	 */
	public Locality getLocality() {
		return locality;
	}

	/**
	 * TODO documentation
	 * @param locality the locality to set.
	 */
	public void setLocality(Locality locality) {
		this.locality = locality;
	}

	/**
	 * TODO documentation
	 * @return returns the hamlet.
	 * @hibernate.property column="`hamlet`" length="250"
	 */
	public String getHamlet() {
		return hamlet;
	}

	/**
	 * TODO documentation
	 * @param hamlet the hamlet to set.
	 */
	public void setHamlet(String hamlet) {
		this.hamlet = hamlet;
	}

	/**
	 * TODO documentation
	 * @return returns the street.
	 * @hibernate.property column="`street`" length="250"
	 */
	public String getStreet() {
		return street;
	}

	/**
	 * TODO documentation
	 * @param street the street to set.
	 */
	public void setStreet(String street) {
		this.street = street;
	}

	/**
	 * TODO documentation
	 * @return returns the zipCode.
	 * @hibernate.property column="`zipCode`" length="25"
	 */
	public String getZipCode() {
		return zipCode;
	}

	/**
	 * TODO documentation
	 * @param zipCode the zipCode to set.
	 */
	public void setZipCode(String zipCode) {
		this.zipCode = zipCode;
	}
    
    /**
     * @see net.smartlab.web.registry.Address#getFormattedString()
     */
    public String getFormattedString() {
        //FIXME use StringBuffer instead of String concat
        String address = this.street != null ? this.street + " " : "";
        address = address + this.civic != null ? this.civic + " " : "";
        address = address + this.zipCode != null ? this.zipCode + " " : "";
        if (this.locality != null) {
            address = address + this.locality.getCity() != null ? this.locality.getCity() + " " : "";
            address = address + this.locality.getState() != null ? this.locality.getState() + " " : "";
            address = address + this.locality.getCountry() != null ? this.locality.getCountry().getDisplay() + " " : "";
        }
        return address;
    }
}