/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.registry;

import java.sql.Date;

import net.smartlab.web.Enumeration;

/**
 * Represents a phisical person registry entry.
 * 
 * @author rlogiacco
 * @hibernate.joined-subclass schema="registry" table="`person`"
 * @hibernate.joined-subclass-key column="`id`"
 */
public class Person extends Entity {

    private final static long serialVersionUID = 6755075929706100845L;

    /**
     * Available genders.
     */
    public final static Gender[] GENDER_LIST = new Gender[] {
            Gender.UNKNOWN, Gender.MALE, Gender.FEMALE
    };

    /**
     * @clientCardinality 1
     * @supplierCardinality 0..1
     */

    private Name name = new Name();

    private Gender gender = Gender.UNKNOWN;

    private Date birthdate;

    /**
     * @clientCardinality 1
     * @supplierCardinality 0..1
     */

    private Locality birthplace = new Locality();

    /**
     * @directed true
     */
    private MailAddress residence;

    private String idCode;

    private boolean privacy;


    /**
     * TODO documentation
     * @hibernate.component
     * @return returns the name.
     */
    public Name getName() {
        return name;
    }

    /**
     * TODO documentation
     * @param name the name to set.
     */
    public void setName(Name name) {
        this.name = name;
    }

    /**
     * @hibernate.property column="`gender`"
     *                     type="net.smartlab.web.registry.Person$Gender"
     * @return Returns the gender.
     */
    public Gender getGender() {
        return gender;
    }

    /**
     * Returns a list of available genders.
     * 
     * @return a list of available genders.
     */
    public Gender[] getGenders() {
        return GENDER_LIST;
    }

    /**
     * @param gender The gender to set.
     */
    public void setGender(Gender gender) {
        this.gender = gender;
    }

    /**
     * TODO documentation
     * @hibernate.property column="`birthdate`"
     * @return returns the birthdate.
     */
    public Date getBirthdate() {
        return birthdate;
    }

    /**
     * TODO documentation
     * @param birthdate the birthdate to set.
     */
    public void setBirthdate(Date birthdate) {
        this.birthdate = birthdate;
    }

    /**
     * TODO documentation
     * @hibernate.component
     * @return returns the birthplace.
     */
    public Locality getBirthplace() {
        return birthplace;
    }

    /**
     * TODO documentation
     * @param birthplace the birthplace to set.
     */
    public void setBirthplace(Locality birthplace) {
        this.birthplace = birthplace;
    }

    /**
     * TODO documentation
     * @hibernate.many-to-one column="`residence`"
     * @return returns the residence.
     */
    public MailAddress getResidence() {
        return residence;
    }

    /**
     * TODO documentation
     * @param residence the residence to set.
     */
    public void setResidence(MailAddress residence) {
        this.residence = residence;
    }

    /**
     * TODO documentation
     * @return returns the idCode.
     * @hibernate.property column="`idCode`"
     */
    public String getIdCode() {
        return idCode;
    }

    /**
     * TODO documentation
     * @param idCode the idCode to set.
     */
    public void setIdCode(String idCode) {
        this.idCode = idCode;
    }


    /**
     * TODO documentation
     * @author rlogiacco
     */
    public static class Gender extends Enumeration {

        private static final long serialVersionUID = -2404603797399493275L;

        /**
         * TODO documentation
         */
        public final static Gender MALE = new Gender('M', "gender.male");

        /**
         * TODO documentation
         */
        public final static Gender FEMALE = new Gender('F', "gender.female");

        /**
         * TODO documentation
         */
        public final static Gender UNKNOWN = new Gender(0, "gender.unknown");


        /**
         * TODO documentation
         */
        public Gender() {
            super();
        }

        /**
         * TODO documentation
         * @param id
         * @param display
         */
        private Gender(int id, String display) {
            super(id, display);
        }

        /**
         * @see net.smartlab.web.Enumeration#decode(int)
         */
        public Enumeration decode(int id) {
            switch (id) {
                case 'M':
                    return MALE;
                case 'F':
                    return FEMALE;
                default:
                    return UNKNOWN;
            }
        }
    }


    /**
     * TODO documentation
     * @return returns the <code>privacy</code>.
     * @hibernate.property column="`privacy`"
     */
    public boolean isPrivacy() {
        return privacy;
    }

    /**
     * TODO documentation
     * @param privacy the <code>privacy</code> to set.
     */
    public void setPrivacy(boolean privacy) {
        this.privacy = privacy;
    }
}
